@@ -4,6 +4,8 @@ import org.json.simple.JSONObject;
 
 import javax.management.MalformedObjectNameException;
 import javax.management.ObjectName;
+import javax.management.remote.JMXServiceURL;
+import java.net.MalformedURLException;
 import java.util.*;
 
 /*
@@ -31,7 +33,8 @@ import java.util.*;
 
 /**
  * Representation of a JMX request which is converted from an GET or POST HTTP
- * Request. A <code>JmxRequest</code> can be obtained only from a {@link org.jmx4perl.JmxRequestFactory}
+ * Request. A <code>JmxRequest</code> can be obtained only from a
+ * {@link org.jmx4perl.JmxRequestFactory}
  *
  * @author roland
  * @since Apr 19, 2009
@@ -74,29 +77,27 @@ public class JmxRequest {
     private List<String> extraArgs;
     private String operation;
     private Type type;
+    private TargetConfig targetConfig = null;
 
     // Max depth of returned JSON structure when deserializing.
     private int maxDepth = 0;
     private int maxCollectionSize = 0;
     private int maxObjects = 0;
 
-
-
-
     /**
      * Create a request with the given type (with no MBean name)
      *
-     * @param pType request's type
+     * @param pType requests type
      */
-    JmxRequest(Type pType) throws MalformedObjectNameException {
-        this(pType,null);
+    JmxRequest(Type pType) {
+        type = pType;
     }
 
     /**
      * Create a request with given type for a certain MBean.
      * Other parameters of the request need to be set explicitely via a setter.
      *
-     * @param pType request's type
+     * @param pType requests type
      * @param pObjectNameS MBean name in string representation
      * @throws MalformedObjectNameException if the name couldnot properly translated
      *         into a JMX {@link javax.management.ObjectName}
@@ -145,6 +146,12 @@ public class JmxRequest {
         if (s != null) {
             operation = s;
         }
+
+        Map target = (Map) pMap.get("target");
+        if (target != null) {
+            targetConfig = new TargetConfig(target);
+        }
+
     }
 
     public String getObjectNameAsString() {
@@ -238,6 +245,10 @@ public class JmxRequest {
         maxDepth = pMaxDepth;
     }
 
+    public TargetConfig getTargetConfig() {
+        return targetConfig;
+    }
+
     @Override
     public String toString() {
         StringBuffer ret = new StringBuffer("JmxRequest[");
@@ -254,6 +265,9 @@ public class JmxRequest {
         if (extraArgs != null && extraArgs.size() > 0) {
             ret.append(", extra=").append(extraArgs);
         }
+        if (targetConfig != null) {
+            ret.append(", target=").append(targetConfig);
+        }
         ret.append("]");
         return ret.toString();
     }
@@ -285,6 +299,55 @@ public class JmxRequest {
         if (operation != null) {
             ret.put("operation",operation);
         }
+        if (targetConfig != null) {
+            ret.put("target", targetConfig.toJSON());
+        }
         return ret;
     }
+
+    // ===============================================================================
+    // Proxy configuration
+
+    public static class TargetConfig {
+        private String url;
+        private Map<String,Object> env;
+
+        public TargetConfig(Map pProxy) {
+            String url = (String) pProxy.get("url");
+            if (url == null) {
+                throw new IllegalArgumentException("No service url given for JSR-160 target");
+            }
+            this.url = url;
+            Map env = (Map) pProxy.get("env");
+            if (env != null) {
+                this.env = env;
+            }
+        }
+
+        public String getUrl() {
+            return url;
+        }
+
+        public Map<String, Object> getEnv() {
+            return env;
+        }
+
+        public JSONObject toJSON() {
+            JSONObject ret = new JSONObject();
+            ret.put("url", url);
+            if (env != null) {
+                ret.put("env", env);
+            }
+            return ret;
+        }
+
+        @Override
+        public String toString() {
+            return "TargetConfig[" +
+                    url +
+                    ", " + env +
+                    "]";
+        }
+    }
+
 }
