@@ -84,6 +84,15 @@ Optional proxy to use
 
 Credentials to use for accessing the proxy
 
+=item target
+
+Add a target which is used for any request served by this object if not already
+a target is present in the request. This way you can setup the default target
+configuration if you are using the agent servlet as a proxy, e.g.
+
+  ... target => { url => "service:jmx:...", user => "...", password => "..." }
+
+
 =back 
 
 =cut
@@ -124,7 +133,6 @@ sub init {
         }
     }
     $self->{ua} = $ua;
-
     return $self;
 }
 
@@ -139,15 +147,13 @@ L<JMX::Jmx4Perl>->request().
 
 sub request {
     my $self = shift;
-    my @jmx_requests = @_;
- 
+    my @jmx_requests = $self->cfg('target') ? $self->_update_targets(@_) : @_;
     my $ua = $self->{ua};
-   
     my $http_req = $self->_to_http_request(@jmx_requests);
     print "Requesting ",$http_req->uri,"\n" if $self->{cfg}->{verbose};
     my $http_resp = $ua->request($http_req);
     my $json_resp = {};
-    print "Response: ",Dumper($http_resp) if $self->{cfg}->{verbose};
+    #print "Response: ",Dumper($http_resp) if $self->{cfg}->{verbose};
     eval {
         $json_resp = from_json($http_resp->content());
     };
@@ -156,7 +162,12 @@ sub request {
         $error_resp->{request} = @jmx_requests > 1 ? undef : $jmx_requests[0];
         return $error_resp;
     }
-    return $self->_from_http_response($json_resp,@jmx_requests);
+    my @responses = ($self->_from_http_response($json_resp,@jmx_requests));
+    if (!wantarray && scalar(@responses) == 1) {
+        return shift @responses;
+    } else {
+        return @responses;
+    }
 }
 
 
@@ -164,7 +175,7 @@ sub request {
 sub _to_http_request {
     my $self = shift;
     my @reqs = @_;
-    if (@reqs == 1) {
+    if (@reqs == 1 && !$reqs[0]->get("target")) {
         # Old, rest-style
         my $url = $self->request_url($reqs[0]);
         return HTTP::Request->new(GET => $url);
@@ -183,14 +194,12 @@ sub _from_http_response {
     my $self = shift;
     my $json_resp = shift;
     my @reqs = @_;
-    if (@reqs == 1) {
-        die "Internal: Not a hash but ",ref($json_resp) unless ref($json_resp) eq "HASH";
+    if (ref($json_resp) eq "HASH") {
         return JMX::Jmx4Perl::Response->new(%{$json_resp},request => $reqs[0]);
-    } else {
-        die "Internal: Not an array of responses but ",ref($json_resp) unless ref($json_resp) eq "ARRAY";
+    } elsif (ref($json_resp) eq "ARRAY") {
         die "Internal: Number of request and responses doesnt match (",scalar(@reqs)," vs. ",scalar(@$json_resp) 
           unless scalar(@reqs) == scalar(@$json_resp);
-
+        
         my @ret = ();        
         for (my $i=0;$i<@reqs;$i++) {
             die "Internal: Not a hash --> ",$json_resp->[$i] unless ref($json_resp->[$i]) eq "HASH";
@@ -198,25 +207,52 @@ sub _from_http_response {
             push @ret,$response;
         }
         return @ret;
+    } else {
+        die "Internal: Not a hash nor an array but ",ref($json_resp) ? ref($json_resp) : $json_resp;
     }
 }
 
+# Update targets if not set in request.
+sub _update_targets {
+    my $self = shift;
+    my @requests = @_;
+    my $target = $self->_clone_target;
+    for my $req (@requests) {
+        $req->{target} = $target unless exists($req->{target});
+        # An request with existing but undefined target removes
+        # any default
+        delete $req->{target} unless defined($req->{target});
+    }
+    return @requests;
+}
+
+sub _clone_target {
+    my $self = shift;
+    die "Internal: No target set" unless $self->cfg('target');
+    my $target = { %{$self->cfg('target')} };
+    if ($target->{env}) {
+        $target->{env} = { %{$target->{env}}};
+    }
+    return $target;
+}
+
 # Validate the JSON answer and the HttpResponse
 sub _validate_response {
     my $self = shift;
-    my ($http_req,$http_resp,$json_resp,$json_error) = @_;
-    
-    if ($json_error && !$http_resp->is_error) {
+    my ($http_req,$http_resp,$json_resp,$json_error) = @_;    
+    if ($json_error) {
         # If not an HTTP-Error and deserialization fails, then we probably 
         # got a wrong URL
         return JMX::Jmx4Perl::Response->new
           ( 
            status => 400,
-           content => $http_resp->content,
-           error => "Error while deserializing JSON answer (probably wrong URL): " . $@
+           error => 
+           "Error while deserializing JSON answer " . (!$http_resp->is_error ? "(Wrong URL ?)" : "") . " : " . $@ . 
+           "\n" . $http_resp->content
           );
     }
-    if ($http_resp->is_error && $json_resp->{status} != 200) {
+    my $sample_resp = $json_resp->[0] if ref($json_resp) eq "ARRAY" && scalar(@$json_resp) == 1;
+    if ($http_resp->is_error && $sample_resp && $sample_resp->{status} == 200) {
         my $error = "Error while fetching ".$http_req->uri." :\n\n" . $http_resp->status_line . "\n";
         my $content = $http_resp->content;
         if ($content) {
@@ -230,7 +266,8 @@ sub _validate_response {
           ( 
            status => $json_resp->{status},
            content => $http_resp->content,
-           error => $error
+           error => $error,
+           stacktrace => $sample_resp->{stacktrace}
           );        
     };
     return undef;
@@ -263,7 +300,7 @@ sub request_url {
         $req .= $self->_extract_path($request->get("path"));
     } elsif ($type eq EXEC) {
         $req .= "/" . $self->_escape($request->get("operation"));
-        $req .= "/" . $self->_escape($self->_null_escape($_)) for @{$request->get("args")};
+        $req .= "/" . $self->_escape($self->_null_escape($_)) for @{$request->get("arguments")};
     } elsif ($type eq SEARCH) {
         # Nothing further to append.
     }
